/*
 * Copyright (c) 2003, 2004 The University of Wroclaw.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *    3. The name of the University may not be used to endorse or promote
 *       products derived from this software without specific prior
 *       written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE UNIVERSITY BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using Nemerle.Compiler;

namespace Nemerle.Compiler {

public class Location 
  {
    public file : string;
    public line : int;
    public column : int;
  }
  
public class Located
  {
    internal static default_loc : Location;

    static this () { default_loc <- Location ("created at compile-time", 0, 0) }

    public mutable loc : Location;
    public this () { loc <- default_loc; }
    public this (loc : Location) { this.loc <- loc }
  }

public variant Modifier 
  {
    | Mod_public
    | Mod_private
    | Mod_new
    | Mod_protected
    | Mod_internal
    | Mod_abstract
    | Mod_virtual
    | Mod_sealed
    | Mod_static
    | Mod_override
    
    | Mod_module
    | Mod_struct
    | Mod_macro
    | Mod_attribute { attrs : list <Parsetree.Expr>; }
  }
  
public variant Literal
  {
    | L_void
    | L_null
    | L_int { val : int; }
    | L_string { val : string; }
    | L_float { val : float; }
    | L_char { val : char; }
    | L_bool { val : bool; }
  }

public variant Fun_body
  {
    | FB_parsed_expr { expr : Parsetree.Expr; }
    | FB_typed_expr { expr : Typedtree.Expr; }
    | FB_compiled_expr
    | FB_extern { name : string; }
    | FB_abstract  // for interafce method
  }

public variant Fun_kind
  {
    | FK_method { f_implements : list <string>; }
    | FK_bound_method { f_implements : list <IMethod>; }
    | FK_ctor
    | FK_static_ctor
    | FK_function
    | FK_iface_method { new_flag : bool; }
  }


} // Nemerle.Compiler
